    package com.croftsoft.apps.tpu;

    import java.util.*;
    import java.util.concurrent.*;

    /***********************************************************************
    * Demonstration of using ThreePhaseUpdate multi-threaded.
    * 
    * For a tutorial on the use of the Three Phase Update (TPU) pattern,
    * please see Croft, "Concurrent Java Simulations Using Three Phase
    * Update", 2009-07-04, <a target="blank"
    * href="http://www.CroftSoft.com/library/tutorials/tpu/">
    *       http://www.CroftSoft.com/library/tutorials/tpu/</a>.
    * 
    * @version
    *   $Id: ThreePhaseUpdateDemo2.java 25 2009-07-04 05:35:25Z croft $
    * @since
    *   2009-06-05
    * @author
    *   <a href="http://www.CroftSoft.com/">David Wallace Croft</a>
    ***********************************************************************/

    public final class  ThreePhaseUpdateDemo2
    ////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////
    {
      
    private static final int
      GRID_WIDTH  = 2,
      GRID_HEIGHT = 2;
    
    ////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////

    public static void  main ( final String [ ]  args )
      throws Exception
    ////////////////////////////////////////////////////////////////////////
    {
      final Random  random = new Random ( );
      
      final ThreePhaseUpdateGrid  grid
        = new ThreePhaseUpdateGrid ( random, GRID_WIDTH, GRID_HEIGHT );
      
      final ThreePhaseUpdate [ ]  threePhaseUpdates = grid.getCells ( );
      
      System.out.println ( "runnerCount = " + threePhaseUpdates.length );
      
      final int  threadCount
        = Runtime.getRuntime ( ).availableProcessors ( );
      
      System.out.println ( "threadCount = " + threadCount );
      
      final ExecutorService  executorService
        = Executors.newFixedThreadPool ( threadCount );
      
      final BlockingQueue<Runnable>  runnableBlockingQueue
        = new LinkedBlockingQueue<Runnable> ( );
      
      final long  startTime = System.currentTimeMillis ( );
      
      for ( int  i = 0; i < 3; i++ )
      {
        update (
          threePhaseUpdates,
          executorService,
          runnableBlockingQueue );
      
        System.out.println ( "update complete" );
      }
      
      System.out.println (
        "elapsed time:  " + ( System.currentTimeMillis ( ) - startTime ) );
      
      executorService.shutdown ( );
    }
    
    public static void  update (
      final ThreePhaseUpdate [ ]     threePhaseUpdates,
      final ExecutorService          executorService,
      final BlockingQueue<Runnable>  runnableBlockingQueue )
      throws Exception
    ////////////////////////////////////////////////////////////////////////
    {
      final int  runnerCount = threePhaseUpdates.length;
      
      final CountDownLatch
        countDownLatch1 = new CountDownLatch ( runnerCount ),
        countDownLatch2 = new CountDownLatch ( runnerCount );
      
      for ( int  i = 0; i < runnerCount; i++ )
      {
        final ThreePhaseUpdate  threePhaseUpdate = threePhaseUpdates [ i ];
        
        executorService.execute (
          new Runnable ( )
          {
            @Override
            public void  run ( )
            {
              try
              {        
                threePhaseUpdate.access ( );
              }
              finally
              {
                countDownLatch1.countDown ( );
              
                try
                {
                  threePhaseUpdate.digest ( );
                }
                finally
                {
                  runnableBlockingQueue.add (
                    new Runnable ( )
                    {
                      @Override
                      public void  run ( )
                      {
                        try
                        {
                          threePhaseUpdate.mutate ( );
                        }
                        finally
                        {                      
                          countDownLatch2.countDown ( );
                        }
                      }
                    } );
                }
              }
            }
          } );
      }
      
      countDownLatch1.await ( );
      
      int  mutateCount = 0;
      
      while ( mutateCount < runnerCount )
      {
        executorService.execute ( runnableBlockingQueue.take ( ) );
        
        mutateCount++;
      }
      
      countDownLatch2.await ( );
    }
    
    ////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////
    }