    package com.croftsoft.apps.tpu;
    
    import java.util.*;

    /***********************************************************************
    * Three phase update demo implementation.
    * 
    * @version
    *   $Id: ThreePhaseUpdateGrid.java 16 2009-06-06 01:01:10Z croft $
    * @since
    *   2009-05-10
    * @author
    *   <a href="http://www.CroftSoft.com/">David Wallace Croft</a>
    ***********************************************************************/

    public final class  ThreePhaseUpdateGrid
    ////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////
    {
      
    private final int
      gridWidth,
      gridHeight;
      
    private final ThreePhaseUpdateCell [ ]  cells;
    
    ////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////
    
    public  ThreePhaseUpdateGrid (
      final Random  random,
      final int     gridWidth,
      final int     gridHeight )
    ////////////////////////////////////////////////////////////////////////
    {
      this.gridWidth  = gridWidth;
      
      this.gridHeight = gridHeight;
      
      cells = new ThreePhaseUpdateCell [ gridHeight * gridWidth ];
      
      for ( int  i = 0; i < cells.length; i++ )
      {
        final String  id = String.format (
          "%1$0" + Integer.toString ( cells.length ).length ( ) + "d",
          Integer.valueOf ( i ) );
        
        cells [ i ] = new ThreePhaseUpdateCell ( id, random );
      }
      
      for ( int  row = 0; row < gridHeight; row++ )
      {
        for ( int  column = 0; column < gridWidth; column++ )
        {
          final ThreePhaseUpdateCell  cell = getCell ( row, column );
          
          final List<ThreePhaseUpdateCell>  adjacentCellList
            = new ArrayList<ThreePhaseUpdateCell> ( 8 );
          
          for ( int  r = row - 1; r <= row + 1; r++ )
          {
            for ( int  c = column - 1; c <= column + 1; c++ )
            {
              if ( r == c )
              {
                continue;
              }
              
              final ThreePhaseUpdateCell  adjacentCell = getCell ( r, c );
              
              if ( adjacentCell != null )
              {
                adjacentCellList.add ( adjacentCell );
              }
            }
          }
          
          cell.setAdjacentCells (
            adjacentCellList.toArray ( new ThreePhaseUpdateCell [ 0 ] ) );
        }
      }
    }
      
    ////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////
    
    public ThreePhaseUpdateCell  getCell (
      final int  row,
      final int  column )
    ////////////////////////////////////////////////////////////////////////
    {
      if ( ( row    < 0           )
        || ( column < 0           )
        || ( row    >= gridHeight )
        || ( column >= gridWidth  ) )
      {
        return null;
      }
      
      return cells [ row * gridWidth + column ];
    }
    
    public ThreePhaseUpdateCell [ ]  getCells ( )
    ////////////////////////////////////////////////////////////////////////
    {
      return cells;
    }
    
    public int  getGridHeight ( )
    ////////////////////////////////////////////////////////////////////////
    {
      return gridHeight;
    }
    
    public int  getGridWidth ( )
    ////////////////////////////////////////////////////////////////////////
    {
      return gridWidth;
    }
    
    ////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////
    }